use crate::util::pack::{pack_u16, unpack_u16};

use super::Packet;

#[derive(Clone)]
pub struct PacketRegister {
    pub handle:String,
    pub secret:Vec<u8>,
    pub code:Vec<u8>,
}
impl PacketRegister {
    pub fn new() -> Self
    {
        Self {
            handle:String::new(),
            secret:Vec::new(),
            code:Vec::new(),
        }
    }
}
impl Packet for PacketRegister {
    type Data = Self;

    fn decode(data:&Vec<u8>, index:&mut usize) -> Result<Self::Data, ()>
    {
        let mut result = Self::new();

        let mut length = unpack_u16(data, index) as usize;
        if length > 0 && data.len() >= *index + length {
            match String::from_utf8(data[*index..*index+length].to_vec()) {
                Ok(text) => {
                    result.handle = text;
                    *index += length;
                }
                Err(_) => { return Err(()) }
            }
        } else { return Err(()); }

        length = unpack_u16(data, index) as usize;
        if length > 0 && data.len() >= *index + length {
            result.secret = data[*index..*index+length].to_vec();
            *index += length;
        } else { return Err(()); }

        length = unpack_u16(data, index) as usize;
        if length > 0 && data.len() >= *index + length {
            result.code = data[*index..*index+length].to_vec();
            *index += length;
        } else { return Err(()); }

        Ok(result)
    }
}


#[derive(Clone)]
pub struct PacketRegisterResponse {
    pub status:u16,
    pub token:[u8; 8],
    pub secret:[u8; 16],
}
impl PacketRegisterResponse {
    pub fn new() -> Self
    {
        Self {
            status:0,
            token:[0; 8],
            secret:[0; 16],
        }
    }
}
impl Packet for PacketRegisterResponse {
    type Data = Self;

    fn encode(&self) -> Vec<u8>
    {
        [
            pack_u16(self.status),
            self.token.to_vec(),
            self.secret.to_vec(),
        ].concat()
    }
}
